(* Read and write standard types from / to stream

    J. Rathlev, IEAP, Uni-Kiel, (rathlev(a)physik.uni-kiel.de)

   Acknowledgements:
     ZLibEx and ZLib routines from http://www.base2ti.com

   The contents of this file may be used under the terms of the
   GNU Lesser General Public License Version 2 or later (the "LGPL")

   Software distributed under this License is distributed on an "AS IS" basis,
   WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
   the specific language governing rights and limitations under the License.

   Vers. 1 - Jan. 2006
   *)

unit ExtStreams;

interface

uses
  Windows, Classes, SysUtils;

type
  TWord = record
    case integer of
    0 : (Both : word);
    1 : (Lo,Hi : byte);
    2 : (Bytes : array [0..1] of Byte);
    end;

  TLongWord = record
    case integer of
    0 : (LongWord : cardinal);
    1 : (Lo,Hi : word);
    2 : (LoL,LoH,HiL,HiH : byte);
    3 : (Bytes : array [0..3] of Byte);
    end;

  TInt64 = record
    case integer of
    0: (AsInt64 : int64);
    1: (Lo,Hi   : Cardinal);
    2: (Cardinals : array [0..1] of Cardinal);
    3: (Words : array [0..3] of Word);
    4: (Bytes : array [0..7] of Byte);
    end;

  // extended file stream functions: read/write several data types
  TReadFileStream = class (TFileStream)
  private
    FBufRead   : cardinal;
    FDelim     : byte;
    function GetEof : boolean;
    function GetDelim : char;
  public
    constructor Create (const FileName : string; Share : Word);
    procedure Reset;
    procedure Seek (Value : int64);
    function ReadByte : byte;
    function ReadChar : char;
    function ReadWord : word;
    function ReadInteger : integer;
    function ReadInt64 : int64;
    procedure ReadRecord (var Buffer; Count : cardinal);
    function ReadString : string;
    property Eof : boolean read GetEof;
    property Delimiter : char read GetDelim;
    end;

  TWriteFileStream = class (TFileStream)
  public
    constructor Create (const FileName : string; Share : Word);
    procedure Reset;
    procedure Seek (Value : int64);
    procedure WriteByte (Value : byte);
    procedure WriteChar (Value : Char);
    procedure WriteWord (Value : word);
    procedure WriteInteger (Value : integer);
    procedure WriteInt64 (Value : int64);
    procedure WriteRecord (const Buffer; Count : cardinal);
    procedure WriteString (s : string);
    procedure WriteStringLine (s : string);
    end;

implementation

{ TReadFileStream ------------------------------------------------------------------- }
constructor TReadFileStream.Create (const FileName : string; Share : Word);
begin
  inherited Create (Filename,fmOpenRead	or Share);
  FDelim:=0;
  end;

procedure TReadFileStream.Reset;
begin
  Position:=0; FDelim:=0;
  end;

procedure TReadFileStream.Seek (Value : int64);
begin
  Position:=Value;
  end;

function TReadFileStream.GetEof : boolean;
begin
  Result:=Position>=Size;
  end;

function TReadFileStream.ReadByte : byte;
begin
  if Read(Result,1)<1 then Result:=0;
  end;

function TReadFileStream.ReadChar : char;
begin
  Result:=chr(ReadByte);
  end;

function TReadFileStream.ReadWord : word;
begin
  if Read(Result,sizeof(word))<sizeof(word) then Result:=0;
  end;

function TReadFileStream.ReadInteger : integer;
begin
  if Read(Result,sizeof(integer))<sizeof(integer) then Result:=0;
  end;

function TReadFileStream.ReadInt64 : int64;
begin
  if Read(Result,sizeof(int64))<sizeof(int64) then Result:=0;
  end;

procedure TReadFileStream.ReadRecord (var Buffer; Count : cardinal);
begin
  Read(Buffer,Count);
  end;

const
  TAB = 9;
  LF = 10;
  CR = 13;
  FF = 12;
  SP = 32;

function TReadFileStream.ReadString : string;
var
  s    : string;
  b    : byte;
  ende : boolean;
begin
  s:='';
  repeat
    ende:=Eof;
    if ende then b:=0
    else begin
      repeat
        b:=ReadByte;      // read bytewise
        until not ((b=LF) and (FDelim=CR));
      FDelim:=0;
      ende:=(b=CR) or (b=LF) or (b=FF) or (b=0);
      if not ende and ((b=TAB) or (b>=SP)) then s:=s+chr(b);
      end
    until ende;
  FDelim:=b;
  Result:=s;
  end;

function TReadFileStream.GetDelim : char;
begin
  Result:=chr(FDelim);
  end;

{ TWriteFileStream ------------------------------------------------------------------- }
constructor TWriteFileStream.Create (const FileName : string; Share : Word);
begin
  inherited Create (Filename,fmCreate or Share);
  end;

procedure TWriteFileStream.Reset;
begin
  Position:=0;
  end;

procedure TWriteFileStream.Seek (Value : int64);
begin
  Position:=Value;
  end;

procedure TWriteFileStream.WriteByte (Value : byte);
begin
  Write (Value,1);
  end;

procedure TWriteFileStream.WriteChar (Value : Char);
begin
  WriteByte(ord(Value));
  end;

procedure TWriteFileStream.WriteWord (Value : word);
begin
  Write (Value,sizeof(word));
  end;

procedure TWriteFileStream.WriteInteger (Value : integer);
begin
  Write (Value,sizeof(integer));
  end;

procedure TWriteFileStream.WriteInt64 (Value : int64);
begin
  Write (Value,sizeof(int64));
  end;

procedure TWriteFileStream.WriteRecord (const Buffer; Count : cardinal);
begin
  Write (Buffer,Count);
  end;

procedure TWriteFileStream.WriteString (s : string);
var
  i   : integer;
begin
  for i:=1 to length(s) do WriteByte (ord(s[i]));
  end;

procedure TWriteFileStream.WriteStringLine (s : string);
begin
  WriteString(s); WriteByte(CR); WriteByte(LF);
  end;

end.
